#include <algorithm>
#include <iostream>
#include <iterator>
#include <numeric>
#include <stdexcept>
#include "Vec.h"

#include "Student_info.h"
#include "znamka.h"
#include "stred.h"

using std::accumulate;           using std::back_inserter;
using std::domain_error;         using std::endl;
using std::ostream;              using std::remove_copy;
using std::transform;

double znamka_pom(const Student_info& s)
{
	try {
		return znamka(s);
	} catch (domain_error) {
		return znamka(s.polovina, s.zaver, 0);
	}
}

// tato verze pracuje sprvn
double stred_analyza(const Vec<Student_info>& studenti)
{
	Vec<double> znamky;

	transform(studenti.begin(), studenti.end(),
	          back_inserter(znamky), znamka_pom);
	return stred(znamky);
}

void vypis_analyza(ostream& vystup, const Str& jmeno,
                   double analyza(const Vec<Student_info>&),
                   const Vec<Student_info>& udelali,
                   const Vec<Student_info>& neudelali)
{
	vystup << jmeno << ": stred(udelali) = " << analyza(udelali) <<
	                   ", stred(neudelali) = " << analyza(neudelali) << endl;
}

double prumer(const Vec<double>& v)
{
	return accumulate(v.begin(), v.end(), 0.0) / v.size();
}

double prumer_znamka(const Student_info& s)
{
	return znamka(s.polovina, s.zaver, prumer(s.domaciukol));
}

double prumer_analyza(const Vec<Student_info>& studenti)
{
	Vec<double> znamky;

	transform(studenti.begin(), studenti.end(),
	          back_inserter(znamky), prumer_znamka);
	return stred(znamky);
}

// stedn hodnota nenulovch prvk 's.domaciukol' nebo 0, pokud dn takov prvek neexistuje
double optimisticky_stred(const Student_info& s)
{
	Vec<double> nenulove;
	remove_copy(s.domaciukol.begin(), s.domaciukol.end(),
	            back_inserter(nenulove), 0);

	if (nenulove.empty())
		return znamka(s.polovina, s.zaver, 0);
	else
		return znamka(s.polovina, s.zaver, stred(nenulove));
}

double optimisticky_stred_analyza(const Vec<Student_info>& studenti)
{
        Vec<double> znamky;

        transform(studenti.begin(), studenti.end(),
                  back_inserter(znamky), optimisticky_stred);
        return stred(znamky);
}